import sys
from datetime import datetime
from typing import List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

ApplicationCode = str
ApplicationDescription = str
ApplicationName = str
BooleanObject = bool
BucketARN = str
ErrorMessage = str
FileKey = str
Id = str
InAppStreamName = str
InAppTableName = str
InputParallelismCount = int
KinesisAnalyticsARN = str
ListApplicationsInputLimit = int
LogStreamARN = str
ParsedInputRecordField = str
ProcessedInputRecord = str
RawInputRecord = str
RecordColumnDelimiter = str
RecordColumnMapping = str
RecordColumnName = str
RecordColumnSqlType = str
RecordEncoding = str
RecordRowDelimiter = str
RecordRowPath = str
ResourceARN = str
RoleARN = str
TagKey = str
TagValue = str


class ApplicationStatus(str):
    DELETING = "DELETING"
    STARTING = "STARTING"
    STOPPING = "STOPPING"
    READY = "READY"
    RUNNING = "RUNNING"
    UPDATING = "UPDATING"


class InputStartingPosition(str):
    NOW = "NOW"
    TRIM_HORIZON = "TRIM_HORIZON"
    LAST_STOPPED_POINT = "LAST_STOPPED_POINT"


class RecordFormatType(str):
    JSON = "JSON"
    CSV = "CSV"


class CodeValidationException(ServiceException):
    """User-provided application code (query) is invalid. This can be a simple
    syntax error.
    """

    code: str = "CodeValidationException"
    sender_fault: bool = False
    status_code: int = 400


class ConcurrentModificationException(ServiceException):
    """Exception thrown as a result of concurrent modification to an
    application. For example, two individuals attempting to edit the same
    application at the same time.
    """

    code: str = "ConcurrentModificationException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidApplicationConfigurationException(ServiceException):
    """User-provided application configuration is not valid."""

    code: str = "InvalidApplicationConfigurationException"
    sender_fault: bool = False
    status_code: int = 400


class InvalidArgumentException(ServiceException):
    """Specified input parameter value is invalid."""

    code: str = "InvalidArgumentException"
    sender_fault: bool = False
    status_code: int = 400


class LimitExceededException(ServiceException):
    """Exceeded the number of applications allowed."""

    code: str = "LimitExceededException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceInUseException(ServiceException):
    """Application is not available for this operation."""

    code: str = "ResourceInUseException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceNotFoundException(ServiceException):
    """Specified application can't be found."""

    code: str = "ResourceNotFoundException"
    sender_fault: bool = False
    status_code: int = 400


class ResourceProvisionedThroughputExceededException(ServiceException):
    """Discovery failed to get a record from the streaming source because of
    the Amazon Kinesis Streams ProvisionedThroughputExceededException. For
    more information, see
    `GetRecords <https://docs.aws.amazon.com/kinesis/latest/APIReference/API_GetRecords.html>`__
    in the Amazon Kinesis Streams API Reference.
    """

    code: str = "ResourceProvisionedThroughputExceededException"
    sender_fault: bool = False
    status_code: int = 400


class ServiceUnavailableException(ServiceException):
    """The service is unavailable. Back off and retry the operation."""

    code: str = "ServiceUnavailableException"
    sender_fault: bool = False
    status_code: int = 400


class TooManyTagsException(ServiceException):
    """Application created with too many tags, or too many tags added to an
    application. Note that the maximum number of application tags includes
    system tags. The maximum number of user-defined application tags is 50.
    """

    code: str = "TooManyTagsException"
    sender_fault: bool = False
    status_code: int = 400


ProcessedInputRecords = List[ProcessedInputRecord]
RawInputRecords = List[RawInputRecord]


class UnableToDetectSchemaException(ServiceException):
    """Data format is not valid. Amazon Kinesis Analytics is not able to detect
    schema for the given streaming source.
    """

    code: str = "UnableToDetectSchemaException"
    sender_fault: bool = False
    status_code: int = 400
    RawInputRecords: Optional[RawInputRecords]
    ProcessedInputRecords: Optional[ProcessedInputRecords]


class UnsupportedOperationException(ServiceException):
    """The request was rejected because a specified parameter is not supported
    or a specified resource is not valid for this operation.
    """

    code: str = "UnsupportedOperationException"
    sender_fault: bool = False
    status_code: int = 400


class CloudWatchLoggingOption(TypedDict, total=False):
    """Provides a description of CloudWatch logging options, including the log
    stream Amazon Resource Name (ARN) and the role ARN.
    """

    LogStreamARN: LogStreamARN
    RoleARN: RoleARN


ApplicationVersionId = int


class AddApplicationCloudWatchLoggingOptionRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    CloudWatchLoggingOption: CloudWatchLoggingOption


class AddApplicationCloudWatchLoggingOptionResponse(TypedDict, total=False):
    pass


class InputLambdaProcessor(TypedDict, total=False):
    """An object that contains the Amazon Resource Name (ARN) of the `AWS
    Lambda <https://docs.aws.amazon.com/lambda/>`__ function that is used to
    preprocess records in the stream, and the ARN of the IAM role that is
    used to access the AWS Lambda function.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class InputProcessingConfiguration(TypedDict, total=False):
    """Provides a description of a processor that is used to preprocess the
    records in the stream before being processed by your application code.
    Currently, the only input processor available is `AWS
    Lambda <https://docs.aws.amazon.com/lambda/>`__.
    """

    InputLambdaProcessor: InputLambdaProcessor


class AddApplicationInputProcessingConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    InputId: Id
    InputProcessingConfiguration: InputProcessingConfiguration


class AddApplicationInputProcessingConfigurationResponse(TypedDict, total=False):
    pass


class RecordColumn(TypedDict, total=False):
    """Describes the mapping of each data element in the streaming source to
    the corresponding column in the in-application stream.

    Also used to describe the format of the reference data source.
    """

    Name: RecordColumnName
    Mapping: Optional[RecordColumnMapping]
    SqlType: RecordColumnSqlType


RecordColumns = List[RecordColumn]


class CSVMappingParameters(TypedDict, total=False):
    """Provides additional mapping information when the record format uses
    delimiters, such as CSV. For example, the following sample records use
    CSV format, where the records use the *'\n'* as the row delimiter and a
    comma (",") as the column delimiter:

    ``"name1", "address1"``

    ``"name2", "address2"``
    """

    RecordRowDelimiter: RecordRowDelimiter
    RecordColumnDelimiter: RecordColumnDelimiter


class JSONMappingParameters(TypedDict, total=False):
    """Provides additional mapping information when JSON is the record format
    on the streaming source.
    """

    RecordRowPath: RecordRowPath


class MappingParameters(TypedDict, total=False):
    """When configuring application input at the time of creating or updating
    an application, provides additional mapping information specific to the
    record format (such as JSON, CSV, or record fields delimited by some
    delimiter) on the streaming source.
    """

    JSONMappingParameters: Optional[JSONMappingParameters]
    CSVMappingParameters: Optional[CSVMappingParameters]


class RecordFormat(TypedDict, total=False):
    """Describes the record format and relevant mapping information that should
    be applied to schematize the records on the stream.
    """

    RecordFormatType: RecordFormatType
    MappingParameters: Optional[MappingParameters]


class SourceSchema(TypedDict, total=False):
    """Describes the format of the data in the streaming source, and how each
    data element maps to corresponding columns created in the in-application
    stream.
    """

    RecordFormat: RecordFormat
    RecordEncoding: Optional[RecordEncoding]
    RecordColumns: RecordColumns


class InputParallelism(TypedDict, total=False):
    """Describes the number of in-application streams to create for a given
    streaming source. For information about parallelism, see `Configuring
    Application
    Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.
    """

    Count: Optional[InputParallelismCount]


class KinesisFirehoseInput(TypedDict, total=False):
    """Identifies an Amazon Kinesis Firehose delivery stream as the streaming
    source. You provide the delivery stream's Amazon Resource Name (ARN) and
    an IAM role ARN that enables Amazon Kinesis Analytics to access the
    stream on your behalf.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class KinesisStreamsInput(TypedDict, total=False):
    """Identifies an Amazon Kinesis stream as the streaming source. You provide
    the stream's Amazon Resource Name (ARN) and an IAM role ARN that enables
    Amazon Kinesis Analytics to access the stream on your behalf.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class Input(TypedDict, total=False):
    """When you configure the application input, you specify the streaming
    source, the in-application stream name that is created, and the mapping
    between the two. For more information, see `Configuring Application
    Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.
    """

    NamePrefix: InAppStreamName
    InputProcessingConfiguration: Optional[InputProcessingConfiguration]
    KinesisStreamsInput: Optional[KinesisStreamsInput]
    KinesisFirehoseInput: Optional[KinesisFirehoseInput]
    InputParallelism: Optional[InputParallelism]
    InputSchema: SourceSchema


class AddApplicationInputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    Input: Input


class AddApplicationInputResponse(TypedDict, total=False):
    pass


class DestinationSchema(TypedDict, total=False):
    """Describes the data format when records are written to the destination.
    For more information, see `Configuring Application
    Output <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html>`__.
    """

    RecordFormatType: RecordFormatType


class LambdaOutput(TypedDict, total=False):
    """When configuring application output, identifies an AWS Lambda function
    as the destination. You provide the function Amazon Resource Name (ARN)
    and also an IAM role ARN that Amazon Kinesis Analytics can use to write
    to the function on your behalf.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class KinesisFirehoseOutput(TypedDict, total=False):
    """When configuring application output, identifies an Amazon Kinesis
    Firehose delivery stream as the destination. You provide the stream
    Amazon Resource Name (ARN) and an IAM role that enables Amazon Kinesis
    Analytics to write to the stream on your behalf.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class KinesisStreamsOutput(TypedDict, total=False):
    """When configuring application output, identifies an Amazon Kinesis stream
    as the destination. You provide the stream Amazon Resource Name (ARN)
    and also an IAM role ARN that Amazon Kinesis Analytics can use to write
    to the stream on your behalf.
    """

    ResourceARN: ResourceARN
    RoleARN: RoleARN


class Output(TypedDict, total=False):
    """Describes application output configuration in which you identify an
    in-application stream and a destination where you want the
    in-application stream data to be written. The destination can be an
    Amazon Kinesis stream or an Amazon Kinesis Firehose delivery stream.

    For limits on how many destinations an application can write and other
    limitations, see
    `Limits <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html>`__.
    """

    Name: InAppStreamName
    KinesisStreamsOutput: Optional[KinesisStreamsOutput]
    KinesisFirehoseOutput: Optional[KinesisFirehoseOutput]
    LambdaOutput: Optional[LambdaOutput]
    DestinationSchema: DestinationSchema


class AddApplicationOutputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    Output: Output


class AddApplicationOutputResponse(TypedDict, total=False):
    pass


class S3ReferenceDataSource(TypedDict, total=False):
    """Identifies the S3 bucket and object that contains the reference data.
    Also identifies the IAM role Amazon Kinesis Analytics can assume to read
    this object on your behalf.

    An Amazon Kinesis Analytics application loads reference data only once.
    If the data changes, you call the
    `UpdateApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_UpdateApplication.html>`__
    operation to trigger reloading of data into your application.
    """

    BucketARN: BucketARN
    FileKey: FileKey
    ReferenceRoleARN: RoleARN


class ReferenceDataSource(TypedDict, total=False):
    """Describes the reference data source by providing the source information
    (S3 bucket name and object key name), the resulting in-application table
    name that is created, and the necessary schema to map the data elements
    in the Amazon S3 object to the in-application table.
    """

    TableName: InAppTableName
    S3ReferenceDataSource: Optional[S3ReferenceDataSource]
    ReferenceSchema: SourceSchema


class AddApplicationReferenceDataSourceRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    ReferenceDataSource: ReferenceDataSource


class AddApplicationReferenceDataSourceResponse(TypedDict, total=False):
    pass


class CloudWatchLoggingOptionDescription(TypedDict, total=False):
    """Description of the CloudWatch logging option."""

    CloudWatchLoggingOptionId: Optional[Id]
    LogStreamARN: LogStreamARN
    RoleARN: RoleARN


CloudWatchLoggingOptionDescriptions = List[CloudWatchLoggingOptionDescription]


class S3ReferenceDataSourceDescription(TypedDict, total=False):
    """Provides the bucket name and object key name that stores the reference
    data.
    """

    BucketARN: BucketARN
    FileKey: FileKey
    ReferenceRoleARN: RoleARN


class ReferenceDataSourceDescription(TypedDict, total=False):
    """Describes the reference data source configured for an application."""

    ReferenceId: Id
    TableName: InAppTableName
    S3ReferenceDataSourceDescription: S3ReferenceDataSourceDescription
    ReferenceSchema: Optional[SourceSchema]


ReferenceDataSourceDescriptions = List[ReferenceDataSourceDescription]


class LambdaOutputDescription(TypedDict, total=False):
    """For an application output, describes the AWS Lambda function configured
    as its destination.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class KinesisFirehoseOutputDescription(TypedDict, total=False):
    """For an application output, describes the Amazon Kinesis Firehose
    delivery stream configured as its destination.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class KinesisStreamsOutputDescription(TypedDict, total=False):
    """For an application output, describes the Amazon Kinesis stream
    configured as its destination.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class OutputDescription(TypedDict, total=False):
    """Describes the application output configuration, which includes the
    in-application stream name and the destination where the stream data is
    written. The destination can be an Amazon Kinesis stream or an Amazon
    Kinesis Firehose delivery stream.
    """

    OutputId: Optional[Id]
    Name: Optional[InAppStreamName]
    KinesisStreamsOutputDescription: Optional[KinesisStreamsOutputDescription]
    KinesisFirehoseOutputDescription: Optional[KinesisFirehoseOutputDescription]
    LambdaOutputDescription: Optional[LambdaOutputDescription]
    DestinationSchema: Optional[DestinationSchema]


OutputDescriptions = List[OutputDescription]


class InputStartingPositionConfiguration(TypedDict, total=False):
    """Describes the point at which the application reads from the streaming
    source.
    """

    InputStartingPosition: Optional[InputStartingPosition]


class KinesisFirehoseInputDescription(TypedDict, total=False):
    """Describes the Amazon Kinesis Firehose delivery stream that is configured
    as the streaming source in the application input configuration.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class KinesisStreamsInputDescription(TypedDict, total=False):
    """Describes the Amazon Kinesis stream that is configured as the streaming
    source in the application input configuration.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class InputLambdaProcessorDescription(TypedDict, total=False):
    """An object that contains the Amazon Resource Name (ARN) of the `AWS
    Lambda <https://docs.aws.amazon.com/lambda/>`__ function that is used to
    preprocess records in the stream, and the ARN of the IAM role that is
    used to access the AWS Lambda expression.
    """

    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]


class InputProcessingConfigurationDescription(TypedDict, total=False):
    """Provides configuration information about an input processor. Currently,
    the only input processor available is `AWS
    Lambda <https://docs.aws.amazon.com/lambda/>`__.
    """

    InputLambdaProcessorDescription: Optional[InputLambdaProcessorDescription]


InAppStreamNames = List[InAppStreamName]


class InputDescription(TypedDict, total=False):
    """Describes the application input configuration. For more information, see
    `Configuring Application
    Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.
    """

    InputId: Optional[Id]
    NamePrefix: Optional[InAppStreamName]
    InAppStreamNames: Optional[InAppStreamNames]
    InputProcessingConfigurationDescription: Optional[InputProcessingConfigurationDescription]
    KinesisStreamsInputDescription: Optional[KinesisStreamsInputDescription]
    KinesisFirehoseInputDescription: Optional[KinesisFirehoseInputDescription]
    InputSchema: Optional[SourceSchema]
    InputParallelism: Optional[InputParallelism]
    InputStartingPositionConfiguration: Optional[InputStartingPositionConfiguration]


InputDescriptions = List[InputDescription]
Timestamp = datetime


class ApplicationDetail(TypedDict, total=False):
    """This documentation is for version 1 of the Amazon Kinesis Data Analytics
    API, which only supports SQL applications. Version 2 of the API supports
    SQL and Java applications. For more information about version 2, see
    `Amazon Kinesis Data Analytics API V2
    Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

    Provides a description of the application, including the application
    Amazon Resource Name (ARN), status, latest version, and input and output
    configuration.
    """

    ApplicationName: ApplicationName
    ApplicationDescription: Optional[ApplicationDescription]
    ApplicationARN: ResourceARN
    ApplicationStatus: ApplicationStatus
    CreateTimestamp: Optional[Timestamp]
    LastUpdateTimestamp: Optional[Timestamp]
    InputDescriptions: Optional[InputDescriptions]
    OutputDescriptions: Optional[OutputDescriptions]
    ReferenceDataSourceDescriptions: Optional[ReferenceDataSourceDescriptions]
    CloudWatchLoggingOptionDescriptions: Optional[CloudWatchLoggingOptionDescriptions]
    ApplicationCode: Optional[ApplicationCode]
    ApplicationVersionId: ApplicationVersionId


class ApplicationSummary(TypedDict, total=False):
    """This documentation is for version 1 of the Amazon Kinesis Data Analytics
    API, which only supports SQL applications. Version 2 of the API supports
    SQL and Java applications. For more information about version 2, see
    `Amazon Kinesis Data Analytics API V2
    Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

    Provides application summary information, including the application
    Amazon Resource Name (ARN), name, and status.
    """

    ApplicationName: ApplicationName
    ApplicationARN: ResourceARN
    ApplicationStatus: ApplicationStatus


ApplicationSummaries = List[ApplicationSummary]


class CloudWatchLoggingOptionUpdate(TypedDict, total=False):
    """Describes CloudWatch logging option updates."""

    CloudWatchLoggingOptionId: Id
    LogStreamARNUpdate: Optional[LogStreamARN]
    RoleARNUpdate: Optional[RoleARN]


CloudWatchLoggingOptionUpdates = List[CloudWatchLoggingOptionUpdate]


class S3ReferenceDataSourceUpdate(TypedDict, total=False):
    """Describes the S3 bucket name, object key name, and IAM role that Amazon
    Kinesis Analytics can assume to read the Amazon S3 object on your behalf
    and populate the in-application reference table.
    """

    BucketARNUpdate: Optional[BucketARN]
    FileKeyUpdate: Optional[FileKey]
    ReferenceRoleARNUpdate: Optional[RoleARN]


class ReferenceDataSourceUpdate(TypedDict, total=False):
    """When you update a reference data source configuration for an
    application, this object provides all the updated values (such as the
    source bucket name and object key name), the in-application table name
    that is created, and updated mapping information that maps the data in
    the Amazon S3 object to the in-application reference table that is
    created.
    """

    ReferenceId: Id
    TableNameUpdate: Optional[InAppTableName]
    S3ReferenceDataSourceUpdate: Optional[S3ReferenceDataSourceUpdate]
    ReferenceSchemaUpdate: Optional[SourceSchema]


ReferenceDataSourceUpdates = List[ReferenceDataSourceUpdate]


class LambdaOutputUpdate(TypedDict, total=False):
    """When updating an output configuration using the
    `UpdateApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_UpdateApplication.html>`__
    operation, provides information about an AWS Lambda function configured
    as the destination.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class KinesisFirehoseOutputUpdate(TypedDict, total=False):
    """When updating an output configuration using the
    `UpdateApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_UpdateApplication.html>`__
    operation, provides information about an Amazon Kinesis Firehose
    delivery stream configured as the destination.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class KinesisStreamsOutputUpdate(TypedDict, total=False):
    """When updating an output configuration using the
    `UpdateApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_UpdateApplication.html>`__
    operation, provides information about an Amazon Kinesis stream
    configured as the destination.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class OutputUpdate(TypedDict, total=False):
    """Describes updates to the output configuration identified by the
    ``OutputId``.
    """

    OutputId: Id
    NameUpdate: Optional[InAppStreamName]
    KinesisStreamsOutputUpdate: Optional[KinesisStreamsOutputUpdate]
    KinesisFirehoseOutputUpdate: Optional[KinesisFirehoseOutputUpdate]
    LambdaOutputUpdate: Optional[LambdaOutputUpdate]
    DestinationSchemaUpdate: Optional[DestinationSchema]


OutputUpdates = List[OutputUpdate]


class InputParallelismUpdate(TypedDict, total=False):
    """Provides updates to the parallelism count."""

    CountUpdate: Optional[InputParallelismCount]


class InputSchemaUpdate(TypedDict, total=False):
    """Describes updates for the application's input schema."""

    RecordFormatUpdate: Optional[RecordFormat]
    RecordEncodingUpdate: Optional[RecordEncoding]
    RecordColumnUpdates: Optional[RecordColumns]


class KinesisFirehoseInputUpdate(TypedDict, total=False):
    """When updating application input configuration, provides information
    about an Amazon Kinesis Firehose delivery stream as the streaming
    source.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class KinesisStreamsInputUpdate(TypedDict, total=False):
    """When updating application input configuration, provides information
    about an Amazon Kinesis stream as the streaming source.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class InputLambdaProcessorUpdate(TypedDict, total=False):
    """Represents an update to the
    `InputLambdaProcessor <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputLambdaProcessor.html>`__
    that is used to preprocess the records in the stream.
    """

    ResourceARNUpdate: Optional[ResourceARN]
    RoleARNUpdate: Optional[RoleARN]


class InputProcessingConfigurationUpdate(TypedDict, total=False):
    """Describes updates to an
    `InputProcessingConfiguration <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html>`__.
    """

    InputLambdaProcessorUpdate: InputLambdaProcessorUpdate


class InputUpdate(TypedDict, total=False):
    """Describes updates to a specific input configuration (identified by the
    ``InputId`` of an application).
    """

    InputId: Id
    NamePrefixUpdate: Optional[InAppStreamName]
    InputProcessingConfigurationUpdate: Optional[InputProcessingConfigurationUpdate]
    KinesisStreamsInputUpdate: Optional[KinesisStreamsInputUpdate]
    KinesisFirehoseInputUpdate: Optional[KinesisFirehoseInputUpdate]
    InputSchemaUpdate: Optional[InputSchemaUpdate]
    InputParallelismUpdate: Optional[InputParallelismUpdate]


InputUpdates = List[InputUpdate]


class ApplicationUpdate(TypedDict, total=False):
    """Describes updates to apply to an existing Amazon Kinesis Analytics
    application.
    """

    InputUpdates: Optional[InputUpdates]
    ApplicationCodeUpdate: Optional[ApplicationCode]
    OutputUpdates: Optional[OutputUpdates]
    ReferenceDataSourceUpdates: Optional[ReferenceDataSourceUpdates]
    CloudWatchLoggingOptionUpdates: Optional[CloudWatchLoggingOptionUpdates]


CloudWatchLoggingOptions = List[CloudWatchLoggingOption]


class Tag(TypedDict, total=False):
    """A key-value pair (the value is optional) that you can define and assign
    to AWS resources. If you specify a tag that already exists, the tag
    value is replaced with the value that you specify in the request. Note
    that the maximum number of application tags includes system tags. The
    maximum number of user-defined application tags is 50. For more
    information, see `Using
    Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-tagging.html>`__.
    """

    Key: TagKey
    Value: Optional[TagValue]


Tags = List[Tag]
Outputs = List[Output]
Inputs = List[Input]


class CreateApplicationRequest(ServiceRequest):
    """TBD"""

    ApplicationName: ApplicationName
    ApplicationDescription: Optional[ApplicationDescription]
    Inputs: Optional[Inputs]
    Outputs: Optional[Outputs]
    CloudWatchLoggingOptions: Optional[CloudWatchLoggingOptions]
    ApplicationCode: Optional[ApplicationCode]
    Tags: Optional[Tags]


class CreateApplicationResponse(TypedDict, total=False):
    """TBD"""

    ApplicationSummary: ApplicationSummary


class DeleteApplicationCloudWatchLoggingOptionRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    CloudWatchLoggingOptionId: Id


class DeleteApplicationCloudWatchLoggingOptionResponse(TypedDict, total=False):
    pass


class DeleteApplicationInputProcessingConfigurationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    InputId: Id


class DeleteApplicationInputProcessingConfigurationResponse(TypedDict, total=False):
    pass


class DeleteApplicationOutputRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    OutputId: Id


class DeleteApplicationOutputResponse(TypedDict, total=False):
    pass


class DeleteApplicationReferenceDataSourceRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    ReferenceId: Id


class DeleteApplicationReferenceDataSourceResponse(TypedDict, total=False):
    pass


class DeleteApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CreateTimestamp: Timestamp


class DeleteApplicationResponse(TypedDict, total=False):
    pass


class DescribeApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName


class DescribeApplicationResponse(TypedDict, total=False):
    ApplicationDetail: ApplicationDetail


class S3Configuration(TypedDict, total=False):
    """Provides a description of an Amazon S3 data source, including the Amazon
    Resource Name (ARN) of the S3 bucket, the ARN of the IAM role that is
    used to access the bucket, and the name of the Amazon S3 object that
    contains the data.
    """

    RoleARN: RoleARN
    BucketARN: BucketARN
    FileKey: FileKey


class DiscoverInputSchemaRequest(ServiceRequest):
    ResourceARN: Optional[ResourceARN]
    RoleARN: Optional[RoleARN]
    InputStartingPositionConfiguration: Optional[InputStartingPositionConfiguration]
    S3Configuration: Optional[S3Configuration]
    InputProcessingConfiguration: Optional[InputProcessingConfiguration]


ParsedInputRecord = List[ParsedInputRecordField]
ParsedInputRecords = List[ParsedInputRecord]


class DiscoverInputSchemaResponse(TypedDict, total=False):
    InputSchema: Optional[SourceSchema]
    ParsedInputRecords: Optional[ParsedInputRecords]
    ProcessedInputRecords: Optional[ProcessedInputRecords]
    RawInputRecords: Optional[RawInputRecords]


class InputConfiguration(TypedDict, total=False):
    """When you start your application, you provide this configuration, which
    identifies the input source and the point in the input source at which
    you want the application to start processing records.
    """

    Id: Id
    InputStartingPositionConfiguration: InputStartingPositionConfiguration


InputConfigurations = List[InputConfiguration]


class ListApplicationsRequest(ServiceRequest):
    Limit: Optional[ListApplicationsInputLimit]
    ExclusiveStartApplicationName: Optional[ApplicationName]


class ListApplicationsResponse(TypedDict, total=False):
    ApplicationSummaries: ApplicationSummaries
    HasMoreApplications: BooleanObject


class ListTagsForResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN


class ListTagsForResourceResponse(TypedDict, total=False):
    Tags: Optional[Tags]


class StartApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    InputConfigurations: InputConfigurations


class StartApplicationResponse(TypedDict, total=False):
    pass


class StopApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName


class StopApplicationResponse(TypedDict, total=False):
    pass


TagKeys = List[TagKey]


class TagResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN
    Tags: Tags


class TagResourceResponse(TypedDict, total=False):
    pass


class UntagResourceRequest(ServiceRequest):
    ResourceARN: KinesisAnalyticsARN
    TagKeys: TagKeys


class UntagResourceResponse(TypedDict, total=False):
    pass


class UpdateApplicationRequest(ServiceRequest):
    ApplicationName: ApplicationName
    CurrentApplicationVersionId: ApplicationVersionId
    ApplicationUpdate: ApplicationUpdate


class UpdateApplicationResponse(TypedDict, total=False):
    pass


class KinesisanalyticsApi:

    service = "kinesisanalytics"
    version = "2015-08-14"

    @handler("AddApplicationCloudWatchLoggingOption")
    def add_application_cloud_watch_logging_option(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        cloud_watch_logging_option: CloudWatchLoggingOption,
    ) -> AddApplicationCloudWatchLoggingOptionResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Adds a CloudWatch log stream to monitor application configuration
        errors. For more information about using CloudWatch log streams with
        Amazon Kinesis Analytics applications, see `Working with Amazon
        CloudWatch
        Logs <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html>`__.

        :param application_name: The Kinesis Analytics application name.
        :param current_application_version_id: The version ID of the Kinesis Analytics application.
        :param cloud_watch_logging_option: Provides the CloudWatch log stream Amazon Resource Name (ARN) and the
        IAM role ARN.
        :returns: AddApplicationCloudWatchLoggingOptionResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("AddApplicationInput")
    def add_application_input(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input: Input,
    ) -> AddApplicationInputResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Adds a streaming source to your Amazon Kinesis application. For
        conceptual information, see `Configuring Application
        Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.

        You can add a streaming source either when you create an application or
        you can use this operation to add a streaming source after you create an
        application. For more information, see
        `CreateApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_CreateApplication.html>`__.

        Any configuration update, including adding a streaming source using this
        operation, results in a new version of the application. You can use the
        `DescribeApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html>`__
        operation to find the current application version.

        This operation requires permissions to perform the
        ``kinesisanalytics:AddApplicationInput`` action.

        :param application_name: Name of your existing Amazon Kinesis Analytics application to which you
        want to add the streaming source.
        :param current_application_version_id: Current version of your Amazon Kinesis Analytics application.
        :param input: The
        `Input <https://docs.
        :returns: AddApplicationInputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises CodeValidationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("AddApplicationInputProcessingConfiguration")
    def add_application_input_processing_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input_id: Id,
        input_processing_configuration: InputProcessingConfiguration,
    ) -> AddApplicationInputProcessingConfigurationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Adds an
        `InputProcessingConfiguration <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html>`__
        to an application. An input processor preprocesses records on the input
        stream before the application's SQL code executes. Currently, the only
        input processor available is `AWS
        Lambda <https://docs.aws.amazon.com/lambda/>`__.

        :param application_name: Name of the application to which you want to add the input processing
        configuration.
        :param current_application_version_id: Version of the application to which you want to add the input processing
        configuration.
        :param input_id: The ID of the input configuration to add the input processing
        configuration to.
        :param input_processing_configuration: The
        `InputProcessingConfiguration <https://docs.
        :returns: AddApplicationInputProcessingConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("AddApplicationOutput")
    def add_application_output(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        output: Output,
    ) -> AddApplicationOutputResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Adds an external destination to your Amazon Kinesis Analytics
        application.

        If you want Amazon Kinesis Analytics to deliver data from an
        in-application stream within your application to an external destination
        (such as an Amazon Kinesis stream, an Amazon Kinesis Firehose delivery
        stream, or an AWS Lambda function), you add the relevant configuration
        to your application using this operation. You can configure one or more
        outputs for your application. Each output configuration maps an
        in-application stream and an external destination.

        You can use one of the output configurations to deliver data from your
        in-application error stream to an external destination so that you can
        analyze the errors. For more information, see `Understanding Application
        Output
        (Destination) <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-output.html>`__.

        Any configuration update, including adding a streaming source using this
        operation, results in a new version of the application. You can use the
        `DescribeApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html>`__
        operation to find the current application version.

        For the limits on the number of application inputs and outputs you can
        configure, see
        `Limits <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html>`__.

        This operation requires permissions to perform the
        ``kinesisanalytics:AddApplicationOutput`` action.

        :param application_name: Name of the application to which you want to add the output
        configuration.
        :param current_application_version_id: Version of the application to which you want to add the output
        configuration.
        :param output: An array of objects, each describing one output configuration.
        :returns: AddApplicationOutputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("AddApplicationReferenceDataSource")
    def add_application_reference_data_source(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        reference_data_source: ReferenceDataSource,
    ) -> AddApplicationReferenceDataSourceResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Adds a reference data source to an existing application.

        Amazon Kinesis Analytics reads reference data (that is, an Amazon S3
        object) and creates an in-application table within your application. In
        the request, you provide the source (S3 bucket name and object key
        name), name of the in-application table to create, and the necessary
        mapping information that describes how data in Amazon S3 object maps to
        columns in the resulting in-application table.

        For conceptual information, see `Configuring Application
        Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.
        For the limits on data sources you can add to your application, see
        `Limits <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/limits.html>`__.

        This operation requires permissions to perform the
        ``kinesisanalytics:AddApplicationOutput`` action.

        :param application_name: Name of an existing application.
        :param current_application_version_id: Version of the application for which you are adding the reference data
        source.
        :param reference_data_source: The reference data source can be an object in your Amazon S3 bucket.
        :returns: AddApplicationReferenceDataSourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("CreateApplication")
    def create_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        application_description: ApplicationDescription = None,
        inputs: Inputs = None,
        outputs: Outputs = None,
        cloud_watch_logging_options: CloudWatchLoggingOptions = None,
        application_code: ApplicationCode = None,
        tags: Tags = None,
    ) -> CreateApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Creates an Amazon Kinesis Analytics application. You can configure each
        application with one streaming source as input, application code to
        process the input, and up to three destinations where you want Amazon
        Kinesis Analytics to write the output data from your application. For an
        overview, see `How it
        Works <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works.html>`__.

        In the input configuration, you map the streaming source to an
        in-application stream, which you can think of as a constantly updating
        table. In the mapping, you must provide a schema for the in-application
        stream and map each data column in the in-application stream to a data
        element in the streaming source.

        Your application code is one or more SQL statements that read input
        data, transform it, and generate output. Your application code can
        create one or more SQL artifacts like SQL streams or pumps.

        In the output configuration, you can configure the application to write
        data from in-application streams created in your applications to up to
        three destinations.

        To read data from your source stream or write data to destination
        streams, Amazon Kinesis Analytics needs your permissions. You grant
        these permissions by creating IAM roles. This operation requires
        permissions to perform the ``kinesisanalytics:CreateApplication``
        action.

        For introductory exercises to create an Amazon Kinesis Analytics
        application, see `Getting
        Started <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/getting-started.html>`__.

        :param application_name: Name of your Amazon Kinesis Analytics application (for example,
        ``sample-app``).
        :param application_description: Summary description of the application.
        :param inputs: Use this parameter to configure the application input.
        :param outputs: You can configure application output to write data from any of the
        in-application streams to up to three destinations.
        :param cloud_watch_logging_options: Use this parameter to configure a CloudWatch log stream to monitor
        application configuration errors.
        :param application_code: One or more SQL statements that read input data, transform it, and
        generate output.
        :param tags: A list of one or more tags to assign to the application.
        :returns: CreateApplicationResponse
        :raises CodeValidationException:
        :raises ResourceInUseException:
        :raises LimitExceededException:
        :raises InvalidArgumentException:
        :raises TooManyTagsException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("DeleteApplication")
    def delete_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        create_timestamp: Timestamp,
    ) -> DeleteApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Deletes the specified application. Amazon Kinesis Analytics halts
        application execution and deletes the application, including any
        application artifacts (such as in-application streams, reference table,
        and application code).

        This operation requires permissions to perform the
        ``kinesisanalytics:DeleteApplication`` action.

        :param application_name: Name of the Amazon Kinesis Analytics application to delete.
        :param create_timestamp: You can use the ``DescribeApplication`` operation to get this value.
        :returns: DeleteApplicationResponse
        :raises ConcurrentModificationException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationCloudWatchLoggingOption")
    def delete_application_cloud_watch_logging_option(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        cloud_watch_logging_option_id: Id,
    ) -> DeleteApplicationCloudWatchLoggingOptionResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Deletes a CloudWatch log stream from an application. For more
        information about using CloudWatch log streams with Amazon Kinesis
        Analytics applications, see `Working with Amazon CloudWatch
        Logs <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/cloudwatch-logs.html>`__.

        :param application_name: The Kinesis Analytics application name.
        :param current_application_version_id: The version ID of the Kinesis Analytics application.
        :param cloud_watch_logging_option_id: The ``CloudWatchLoggingOptionId`` of the CloudWatch logging option to
        delete.
        :returns: DeleteApplicationCloudWatchLoggingOptionResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationInputProcessingConfiguration")
    def delete_application_input_processing_configuration(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        input_id: Id,
    ) -> DeleteApplicationInputProcessingConfigurationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Deletes an
        `InputProcessingConfiguration <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_InputProcessingConfiguration.html>`__
        from an input.

        :param application_name: The Kinesis Analytics application name.
        :param current_application_version_id: The version ID of the Kinesis Analytics application.
        :param input_id: The ID of the input configuration from which to delete the input
        processing configuration.
        :returns: DeleteApplicationInputProcessingConfigurationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationOutput")
    def delete_application_output(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        output_id: Id,
    ) -> DeleteApplicationOutputResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Deletes output destination configuration from your application
        configuration. Amazon Kinesis Analytics will no longer write data from
        the corresponding in-application stream to the external output
        destination.

        This operation requires permissions to perform the
        ``kinesisanalytics:DeleteApplicationOutput`` action.

        :param application_name: Amazon Kinesis Analytics application name.
        :param current_application_version_id: Amazon Kinesis Analytics application version.
        :param output_id: The ID of the configuration to delete.
        :returns: DeleteApplicationOutputResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DeleteApplicationReferenceDataSource")
    def delete_application_reference_data_source(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        reference_id: Id,
    ) -> DeleteApplicationReferenceDataSourceResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Deletes a reference data source configuration from the specified
        application configuration.

        If the application is running, Amazon Kinesis Analytics immediately
        removes the in-application table that you created using the
        `AddApplicationReferenceDataSource <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_AddApplicationReferenceDataSource.html>`__
        operation.

        This operation requires permissions to perform the
        ``kinesisanalytics.DeleteApplicationReferenceDataSource`` action.

        :param application_name: Name of an existing application.
        :param current_application_version_id: Version of the application.
        :param reference_id: ID of the reference data source.
        :returns: DeleteApplicationReferenceDataSourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DescribeApplication")
    def describe_application(
        self, context: RequestContext, application_name: ApplicationName
    ) -> DescribeApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Returns information about a specific Amazon Kinesis Analytics
        application.

        If you want to retrieve a list of all applications in your account, use
        the
        `ListApplications <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_ListApplications.html>`__
        operation.

        This operation requires permissions to perform the
        ``kinesisanalytics:DescribeApplication`` action. You can use
        ``DescribeApplication`` to get the current application versionId, which
        you need to call other operations such as ``Update``.

        :param application_name: Name of the application.
        :returns: DescribeApplicationResponse
        :raises ResourceNotFoundException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("DiscoverInputSchema")
    def discover_input_schema(
        self,
        context: RequestContext,
        resource_arn: ResourceARN = None,
        role_arn: RoleARN = None,
        input_starting_position_configuration: InputStartingPositionConfiguration = None,
        s3_configuration: S3Configuration = None,
        input_processing_configuration: InputProcessingConfiguration = None,
    ) -> DiscoverInputSchemaResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Infers a schema by evaluating sample records on the specified streaming
        source (Amazon Kinesis stream or Amazon Kinesis Firehose delivery
        stream) or S3 object. In the response, the operation returns the
        inferred schema and also the sample records that the operation used to
        infer the schema.

        You can use the inferred schema when configuring a streaming source for
        your application. For conceptual information, see `Configuring
        Application
        Input <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-it-works-input.html>`__.
        Note that when you create an application using the Amazon Kinesis
        Analytics console, the console uses this operation to infer a schema and
        show it in the console user interface.

        This operation requires permissions to perform the
        ``kinesisanalytics:DiscoverInputSchema`` action.

        :param resource_arn: Amazon Resource Name (ARN) of the streaming source.
        :param role_arn: ARN of the IAM role that Amazon Kinesis Analytics can assume to access
        the stream on your behalf.
        :param input_starting_position_configuration: Point at which you want Amazon Kinesis Analytics to start reading
        records from the specified streaming source discovery purposes.
        :param s3_configuration: Specify this parameter to discover a schema from data in an Amazon S3
        object.
        :param input_processing_configuration: The
        `InputProcessingConfiguration <https://docs.
        :returns: DiscoverInputSchemaResponse
        :raises InvalidArgumentException:
        :raises UnableToDetectSchemaException:
        :raises ResourceProvisionedThroughputExceededException:
        :raises ServiceUnavailableException:
        """
        raise NotImplementedError

    @handler("ListApplications")
    def list_applications(
        self,
        context: RequestContext,
        limit: ListApplicationsInputLimit = None,
        exclusive_start_application_name: ApplicationName = None,
    ) -> ListApplicationsResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Returns a list of Amazon Kinesis Analytics applications in your account.
        For each application, the response includes the application name, Amazon
        Resource Name (ARN), and status. If the response returns the
        ``HasMoreApplications`` value as true, you can send another request by
        adding the ``ExclusiveStartApplicationName`` in the request body, and
        set the value of this to the last application name from the previous
        response.

        If you want detailed information about a specific application, use
        `DescribeApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html>`__.

        This operation requires permissions to perform the
        ``kinesisanalytics:ListApplications`` action.

        :param limit: Maximum number of applications to list.
        :param exclusive_start_application_name: Name of the application to start the list with.
        :returns: ListApplicationsResponse
        """
        raise NotImplementedError

    @handler("ListTagsForResource")
    def list_tags_for_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN
    ) -> ListTagsForResourceResponse:
        """Retrieves the list of key-value tags assigned to the application. For
        more information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-tagging.html>`__.

        :param resource_arn: The ARN of the application for which to retrieve tags.
        :returns: ListTagsForResourceResponse
        :raises ResourceNotFoundException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("StartApplication")
    def start_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        input_configurations: InputConfigurations,
    ) -> StartApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Starts the specified Amazon Kinesis Analytics application. After
        creating an application, you must exclusively call this operation to
        start your application.

        After the application starts, it begins consuming the input data,
        processes it, and writes the output to the configured destination.

        The application status must be ``READY`` for you to start an
        application. You can get the application status in the console or using
        the
        `DescribeApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html>`__
        operation.

        After you start the application, you can stop the application from
        processing the input by calling the
        `StopApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_StopApplication.html>`__
        operation.

        This operation requires permissions to perform the
        ``kinesisanalytics:StartApplication`` action.

        :param application_name: Name of the application.
        :param input_configurations: Identifies the specific input, by ID, that the application starts
        consuming.
        :returns: StartApplicationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises InvalidApplicationConfigurationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("StopApplication")
    def stop_application(
        self, context: RequestContext, application_name: ApplicationName
    ) -> StopApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Stops the application from processing input data. You can stop an
        application only if it is in the running state. You can use the
        `DescribeApplication <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/API_DescribeApplication.html>`__
        operation to find the application state. After the application is
        stopped, Amazon Kinesis Analytics stops reading data from the input, the
        application stops processing data, and there is no output written to the
        destination.

        This operation requires permissions to perform the
        ``kinesisanalytics:StopApplication`` action.

        :param application_name: Name of the running application to stop.
        :returns: StopApplicationResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError

    @handler("TagResource")
    def tag_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN, tags: Tags
    ) -> TagResourceResponse:
        """Adds one or more key-value tags to a Kinesis Analytics application. Note
        that the maximum number of application tags includes system tags. The
        maximum number of user-defined application tags is 50. For more
        information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-tagging.html>`__.

        :param resource_arn: The ARN of the application to assign the tags.
        :param tags: The key-value tags to assign to the application.
        :returns: TagResourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises TooManyTagsException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("UntagResource")
    def untag_resource(
        self, context: RequestContext, resource_arn: KinesisAnalyticsARN, tag_keys: TagKeys
    ) -> UntagResourceResponse:
        """Removes one or more tags from a Kinesis Analytics application. For more
        information, see `Using
        Tagging <https://docs.aws.amazon.com/kinesisanalytics/latest/dev/how-tagging.html>`__.

        :param resource_arn: The ARN of the Kinesis Analytics application from which to remove the
        tags.
        :param tag_keys: A list of keys of tags to remove from the specified application.
        :returns: UntagResourceResponse
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises TooManyTagsException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        """
        raise NotImplementedError

    @handler("UpdateApplication")
    def update_application(
        self,
        context: RequestContext,
        application_name: ApplicationName,
        current_application_version_id: ApplicationVersionId,
        application_update: ApplicationUpdate,
    ) -> UpdateApplicationResponse:
        """This documentation is for version 1 of the Amazon Kinesis Data Analytics
        API, which only supports SQL applications. Version 2 of the API supports
        SQL and Java applications. For more information about version 2, see
        `Amazon Kinesis Data Analytics API V2
        Documentation </kinesisanalytics/latest/apiv2/Welcome.html>`__.

        Updates an existing Amazon Kinesis Analytics application. Using this
        API, you can update application code, input configuration, and output
        configuration.

        Note that Amazon Kinesis Analytics updates the
        ``CurrentApplicationVersionId`` each time you update your application.

        This operation requires permission for the
        ``kinesisanalytics:UpdateApplication`` action.

        :param application_name: Name of the Amazon Kinesis Analytics application to update.
        :param current_application_version_id: The current application version ID.
        :param application_update: Describes application updates.
        :returns: UpdateApplicationResponse
        :raises CodeValidationException:
        :raises ResourceNotFoundException:
        :raises ResourceInUseException:
        :raises InvalidArgumentException:
        :raises ConcurrentModificationException:
        :raises UnsupportedOperationException:
        """
        raise NotImplementedError



import sys
from datetime import datetime
from typing import Dict, List, Optional

if sys.version_info >= (3, 8):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

from localstack.aws.api import RequestContext, ServiceException, ServiceRequest, handler

MaxResults = int
_boolean = bool
_double = float
_integer = int
_integerMin5Max100 = int
_string = str


class AuthenticationStrategy(str):
    SIMPLE = "SIMPLE"
    LDAP = "LDAP"


class BrokerState(str):
    CREATION_IN_PROGRESS = "CREATION_IN_PROGRESS"
    CREATION_FAILED = "CREATION_FAILED"
    DELETION_IN_PROGRESS = "DELETION_IN_PROGRESS"
    RUNNING = "RUNNING"
    REBOOT_IN_PROGRESS = "REBOOT_IN_PROGRESS"
    CRITICAL_ACTION_REQUIRED = "CRITICAL_ACTION_REQUIRED"


class BrokerStorageType(str):
    EBS = "EBS"
    EFS = "EFS"


class ChangeType(str):
    CREATE = "CREATE"
    UPDATE = "UPDATE"
    DELETE = "DELETE"


class DayOfWeek(str):
    MONDAY = "MONDAY"
    TUESDAY = "TUESDAY"
    WEDNESDAY = "WEDNESDAY"
    THURSDAY = "THURSDAY"
    FRIDAY = "FRIDAY"
    SATURDAY = "SATURDAY"
    SUNDAY = "SUNDAY"


class DeploymentMode(str):
    SINGLE_INSTANCE = "SINGLE_INSTANCE"
    ACTIVE_STANDBY_MULTI_AZ = "ACTIVE_STANDBY_MULTI_AZ"
    CLUSTER_MULTI_AZ = "CLUSTER_MULTI_AZ"


class EngineType(str):
    ACTIVEMQ = "ACTIVEMQ"
    RABBITMQ = "RABBITMQ"


class SanitizationWarningReason(str):
    DISALLOWED_ELEMENT_REMOVED = "DISALLOWED_ELEMENT_REMOVED"
    DISALLOWED_ATTRIBUTE_REMOVED = "DISALLOWED_ATTRIBUTE_REMOVED"
    INVALID_ATTRIBUTE_VALUE_REMOVED = "INVALID_ATTRIBUTE_VALUE_REMOVED"


class BadRequestException(ServiceException):
    """Returns information about an error."""

    code: str = "BadRequestException"
    sender_fault: bool = False
    status_code: int = 400
    ErrorAttribute: Optional[_string]


class ConflictException(ServiceException):
    """Returns information about an error."""

    code: str = "ConflictException"
    sender_fault: bool = False
    status_code: int = 409
    ErrorAttribute: Optional[_string]


class ForbiddenException(ServiceException):
    """Returns information about an error."""

    code: str = "ForbiddenException"
    sender_fault: bool = False
    status_code: int = 403
    ErrorAttribute: Optional[_string]


class InternalServerErrorException(ServiceException):
    """Returns information about an error."""

    code: str = "InternalServerErrorException"
    sender_fault: bool = False
    status_code: int = 500
    ErrorAttribute: Optional[_string]


class NotFoundException(ServiceException):
    """Returns information about an error."""

    code: str = "NotFoundException"
    sender_fault: bool = False
    status_code: int = 404
    ErrorAttribute: Optional[_string]


class UnauthorizedException(ServiceException):
    """Returns information about an error."""

    code: str = "UnauthorizedException"
    sender_fault: bool = False
    status_code: int = 401
    ErrorAttribute: Optional[_string]


class ActionRequired(TypedDict, total=False):
    """The action required to resolve a broker issue when the broker is in a
    CRITICAL_ACTION_REQUIRED state.
    """

    ActionRequiredCode: Optional[_string]
    ActionRequiredInfo: Optional[_string]


class AvailabilityZone(TypedDict, total=False):
    """Name of the availability zone."""

    Name: Optional[_string]


class EngineVersion(TypedDict, total=False):
    """Id of the engine version."""

    Name: Optional[_string]


_listOfEngineVersion = List[EngineVersion]


class BrokerEngineType(TypedDict, total=False):
    """Types of broker engines."""

    EngineType: Optional[EngineType]
    EngineVersions: Optional[_listOfEngineVersion]


_listOfBrokerEngineType = List[BrokerEngineType]


class BrokerEngineTypeOutput(TypedDict, total=False):
    """Returns a list of broker engine type."""

    BrokerEngineTypes: Optional[_listOfBrokerEngineType]
    MaxResults: _integerMin5Max100
    NextToken: Optional[_string]


_listOf__string = List[_string]


class BrokerInstance(TypedDict, total=False):
    """Returns information about all brokers."""

    ConsoleURL: Optional[_string]
    Endpoints: Optional[_listOf__string]
    IpAddress: Optional[_string]


_listOfDeploymentMode = List[DeploymentMode]
_listOfAvailabilityZone = List[AvailabilityZone]


class BrokerInstanceOption(TypedDict, total=False):
    """Option for host instance type."""

    AvailabilityZones: Optional[_listOfAvailabilityZone]
    EngineType: Optional[EngineType]
    HostInstanceType: Optional[_string]
    StorageType: Optional[BrokerStorageType]
    SupportedDeploymentModes: Optional[_listOfDeploymentMode]
    SupportedEngineVersions: Optional[_listOf__string]


_listOfBrokerInstanceOption = List[BrokerInstanceOption]


class BrokerInstanceOptionsOutput(TypedDict, total=False):
    """Returns a list of broker instance options."""

    BrokerInstanceOptions: Optional[_listOfBrokerInstanceOption]
    MaxResults: _integerMin5Max100
    NextToken: Optional[_string]


_timestampIso8601 = datetime


class BrokerSummary(TypedDict, total=False):
    """Returns information about all brokers."""

    BrokerArn: Optional[_string]
    BrokerId: Optional[_string]
    BrokerName: Optional[_string]
    BrokerState: Optional[BrokerState]
    Created: Optional[_timestampIso8601]
    DeploymentMode: DeploymentMode
    EngineType: EngineType
    HostInstanceType: Optional[_string]


_mapOf__string = Dict[_string, _string]


class ConfigurationRevision(TypedDict, total=False):
    """Returns information about the specified configuration revision."""

    Created: _timestampIso8601
    Description: Optional[_string]
    Revision: _integer


class Configuration(TypedDict, total=False):
    """Returns information about all configurations."""

    Arn: _string
    AuthenticationStrategy: AuthenticationStrategy
    Created: _timestampIso8601
    Description: _string
    EngineType: EngineType
    EngineVersion: _string
    Id: _string
    LatestRevision: ConfigurationRevision
    Name: _string
    Tags: Optional[_mapOf__string]


class ConfigurationId(TypedDict, total=False):
    """A list of information about the configuration.

    Does not apply to RabbitMQ brokers.
    """

    Id: _string
    Revision: Optional[_integer]


_listOfConfigurationId = List[ConfigurationId]


class Configurations(TypedDict, total=False):
    """Broker configuration information"""

    Current: Optional[ConfigurationId]
    History: Optional[_listOfConfigurationId]
    Pending: Optional[ConfigurationId]


class User(TypedDict, total=False):
    """A user associated with the broker. For RabbitMQ brokers, one and only
    one administrative user is accepted and created when a broker is first
    provisioned. All subsequent broker users are created by making RabbitMQ
    API calls directly to brokers or via the RabbitMQ web console.
    """

    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Password: _string
    Username: _string


_listOfUser = List[User]


class WeeklyStartTime(TypedDict, total=False):
    """The scheduled time period relative to UTC during which Amazon MQ begins
    to apply pending updates or patches to the broker.
    """

    DayOfWeek: DayOfWeek
    TimeOfDay: _string
    TimeZone: Optional[_string]


class Logs(TypedDict, total=False):
    """The list of information about logs to be enabled for the specified
    broker.
    """

    Audit: Optional[_boolean]
    General: Optional[_boolean]


class LdapServerMetadataInput(TypedDict, total=False):
    """Optional. The metadata of the LDAP server used to authenticate and
    authorize connections to the broker.

    Does not apply to RabbitMQ brokers.
    """

    Hosts: _listOf__string
    RoleBase: _string
    RoleName: Optional[_string]
    RoleSearchMatching: _string
    RoleSearchSubtree: Optional[_boolean]
    ServiceAccountPassword: _string
    ServiceAccountUsername: _string
    UserBase: _string
    UserRoleName: Optional[_string]
    UserSearchMatching: _string
    UserSearchSubtree: Optional[_boolean]


class EncryptionOptions(TypedDict, total=False):
    """Does not apply to RabbitMQ brokers.

    Encryption options for the broker.
    """

    KmsKeyId: Optional[_string]
    UseAwsOwnedKey: _boolean


class CreateBrokerInput(TypedDict, total=False):
    """Creates a broker."""

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: _boolean
    BrokerName: _string
    Configuration: Optional[ConfigurationId]
    CreatorRequestId: Optional[_string]
    DeploymentMode: DeploymentMode
    EncryptionOptions: Optional[EncryptionOptions]
    EngineType: EngineType
    EngineVersion: _string
    HostInstanceType: _string
    LdapServerMetadata: Optional[LdapServerMetadataInput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    PubliclyAccessible: _boolean
    SecurityGroups: Optional[_listOf__string]
    StorageType: Optional[BrokerStorageType]
    SubnetIds: Optional[_listOf__string]
    Tags: Optional[_mapOf__string]
    Users: _listOfUser


class CreateBrokerOutput(TypedDict, total=False):
    """Returns information about the created broker."""

    BrokerArn: Optional[_string]
    BrokerId: Optional[_string]


class CreateBrokerRequest(ServiceRequest):
    """Creates a broker using the specified properties."""

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: _boolean
    BrokerName: _string
    Configuration: Optional[ConfigurationId]
    CreatorRequestId: Optional[_string]
    DeploymentMode: DeploymentMode
    EncryptionOptions: Optional[EncryptionOptions]
    EngineType: EngineType
    EngineVersion: _string
    HostInstanceType: _string
    LdapServerMetadata: Optional[LdapServerMetadataInput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    PubliclyAccessible: _boolean
    SecurityGroups: Optional[_listOf__string]
    StorageType: Optional[BrokerStorageType]
    SubnetIds: Optional[_listOf__string]
    Tags: Optional[_mapOf__string]
    Users: _listOfUser


class CreateBrokerResponse(TypedDict, total=False):
    BrokerArn: Optional[_string]
    BrokerId: Optional[_string]


class CreateConfigurationInput(TypedDict, total=False):
    """Creates a new configuration for the specified configuration name. Amazon
    MQ uses the default configuration (the engine type and version).
    """

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    EngineType: EngineType
    EngineVersion: _string
    Name: _string
    Tags: Optional[_mapOf__string]


class CreateConfigurationOutput(TypedDict, total=False):
    """Returns information about the created configuration."""

    Arn: _string
    AuthenticationStrategy: AuthenticationStrategy
    Created: _timestampIso8601
    Id: _string
    LatestRevision: Optional[ConfigurationRevision]
    Name: _string


class CreateConfigurationRequest(ServiceRequest):
    """Creates a new configuration for the specified configuration name. Amazon
    MQ uses the default configuration (the engine type and version).
    """

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    EngineType: EngineType
    EngineVersion: _string
    Name: _string
    Tags: Optional[_mapOf__string]


class CreateConfigurationResponse(TypedDict, total=False):
    Arn: Optional[_string]
    AuthenticationStrategy: Optional[AuthenticationStrategy]
    Created: Optional[_timestampIso8601]
    Id: Optional[_string]
    LatestRevision: Optional[ConfigurationRevision]
    Name: Optional[_string]


class CreateTagsRequest(ServiceRequest):
    """A map of the key-value pairs for the resource tag."""

    ResourceArn: _string
    Tags: Optional[_mapOf__string]


class CreateUserInput(TypedDict, total=False):
    """Creates a new ActiveMQ user."""

    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Password: _string


class CreateUserRequest(ServiceRequest):
    """Creates a new ActiveMQ user."""

    BrokerId: _string
    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Password: _string
    Username: _string


class CreateUserResponse(TypedDict, total=False):
    pass


class DeleteBrokerOutput(TypedDict, total=False):
    """Returns information about the deleted broker."""

    BrokerId: Optional[_string]


class DeleteBrokerRequest(ServiceRequest):
    BrokerId: _string


class DeleteBrokerResponse(TypedDict, total=False):
    BrokerId: Optional[_string]


class DeleteTagsRequest(ServiceRequest):
    ResourceArn: _string
    TagKeys: _listOf__string


class DeleteUserRequest(ServiceRequest):
    BrokerId: _string
    Username: _string


class DeleteUserResponse(TypedDict, total=False):
    pass


class DescribeBrokerEngineTypesRequest(ServiceRequest):
    EngineType: Optional[_string]
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]


class DescribeBrokerEngineTypesResponse(TypedDict, total=False):
    BrokerEngineTypes: Optional[_listOfBrokerEngineType]
    MaxResults: Optional[_integerMin5Max100]
    NextToken: Optional[_string]


class DescribeBrokerInstanceOptionsRequest(ServiceRequest):
    EngineType: Optional[_string]
    HostInstanceType: Optional[_string]
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]
    StorageType: Optional[_string]


class DescribeBrokerInstanceOptionsResponse(TypedDict, total=False):
    BrokerInstanceOptions: Optional[_listOfBrokerInstanceOption]
    MaxResults: Optional[_integerMin5Max100]
    NextToken: Optional[_string]


class UserSummary(TypedDict, total=False):
    """Returns a list of all broker users. Does not apply to RabbitMQ brokers."""

    PendingChange: Optional[ChangeType]
    Username: _string


_listOfUserSummary = List[UserSummary]


class LdapServerMetadataOutput(TypedDict, total=False):
    """Optional. The metadata of the LDAP server used to authenticate and
    authorize connections to the broker.
    """

    Hosts: _listOf__string
    RoleBase: _string
    RoleName: Optional[_string]
    RoleSearchMatching: _string
    RoleSearchSubtree: Optional[_boolean]
    ServiceAccountUsername: _string
    UserBase: _string
    UserRoleName: Optional[_string]
    UserSearchMatching: _string
    UserSearchSubtree: Optional[_boolean]


class PendingLogs(TypedDict, total=False):
    """The list of information about logs to be enabled for the specified
    broker.
    """

    Audit: Optional[_boolean]
    General: Optional[_boolean]


class LogsSummary(TypedDict, total=False):
    """The list of information about logs currently enabled and pending to be
    deployed for the specified broker.
    """

    Audit: Optional[_boolean]
    AuditLogGroup: Optional[_string]
    General: _boolean
    GeneralLogGroup: _string
    Pending: Optional[PendingLogs]


_listOfBrokerInstance = List[BrokerInstance]
_listOfActionRequired = List[ActionRequired]


class DescribeBrokerOutput(TypedDict, total=False):
    """Returns information about the specified broker."""

    ActionsRequired: Optional[_listOfActionRequired]
    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: _boolean
    BrokerArn: Optional[_string]
    BrokerId: Optional[_string]
    BrokerInstances: Optional[_listOfBrokerInstance]
    BrokerName: Optional[_string]
    BrokerState: Optional[BrokerState]
    Configurations: Optional[Configurations]
    Created: Optional[_timestampIso8601]
    DeploymentMode: DeploymentMode
    EncryptionOptions: Optional[EncryptionOptions]
    EngineType: EngineType
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataOutput]
    Logs: Optional[LogsSummary]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    PendingAuthenticationStrategy: Optional[AuthenticationStrategy]
    PendingEngineVersion: Optional[_string]
    PendingHostInstanceType: Optional[_string]
    PendingLdapServerMetadata: Optional[LdapServerMetadataOutput]
    PendingSecurityGroups: Optional[_listOf__string]
    PubliclyAccessible: _boolean
    SecurityGroups: Optional[_listOf__string]
    StorageType: Optional[BrokerStorageType]
    SubnetIds: Optional[_listOf__string]
    Tags: Optional[_mapOf__string]
    Users: Optional[_listOfUserSummary]


class DescribeBrokerRequest(ServiceRequest):
    BrokerId: _string


class DescribeBrokerResponse(TypedDict, total=False):
    ActionsRequired: Optional[_listOfActionRequired]
    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: Optional[_boolean]
    BrokerArn: Optional[_string]
    BrokerId: Optional[_string]
    BrokerInstances: Optional[_listOfBrokerInstance]
    BrokerName: Optional[_string]
    BrokerState: Optional[BrokerState]
    Configurations: Optional[Configurations]
    Created: Optional[_timestampIso8601]
    DeploymentMode: Optional[DeploymentMode]
    EncryptionOptions: Optional[EncryptionOptions]
    EngineType: Optional[EngineType]
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataOutput]
    Logs: Optional[LogsSummary]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    PendingAuthenticationStrategy: Optional[AuthenticationStrategy]
    PendingEngineVersion: Optional[_string]
    PendingHostInstanceType: Optional[_string]
    PendingLdapServerMetadata: Optional[LdapServerMetadataOutput]
    PendingSecurityGroups: Optional[_listOf__string]
    PubliclyAccessible: Optional[_boolean]
    SecurityGroups: Optional[_listOf__string]
    StorageType: Optional[BrokerStorageType]
    SubnetIds: Optional[_listOf__string]
    Tags: Optional[_mapOf__string]
    Users: Optional[_listOfUserSummary]


class DescribeConfigurationRequest(ServiceRequest):
    ConfigurationId: _string


class DescribeConfigurationResponse(TypedDict, total=False):
    Arn: Optional[_string]
    AuthenticationStrategy: Optional[AuthenticationStrategy]
    Created: Optional[_timestampIso8601]
    Description: Optional[_string]
    EngineType: Optional[EngineType]
    EngineVersion: Optional[_string]
    Id: Optional[_string]
    LatestRevision: Optional[ConfigurationRevision]
    Name: Optional[_string]
    Tags: Optional[_mapOf__string]


class DescribeConfigurationRevisionOutput(TypedDict, total=False):
    """Returns the specified configuration revision for the specified
    configuration.
    """

    ConfigurationId: _string
    Created: _timestampIso8601
    Data: _string
    Description: Optional[_string]


class DescribeConfigurationRevisionRequest(ServiceRequest):
    ConfigurationId: _string
    ConfigurationRevision: _string


class DescribeConfigurationRevisionResponse(TypedDict, total=False):
    ConfigurationId: Optional[_string]
    Created: Optional[_timestampIso8601]
    Data: Optional[_string]
    Description: Optional[_string]


class UserPendingChanges(TypedDict, total=False):
    """Returns information about the status of the changes pending for the
    ActiveMQ user.
    """

    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    PendingChange: ChangeType


class DescribeUserOutput(TypedDict, total=False):
    """Returns information about an ActiveMQ user."""

    BrokerId: _string
    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Pending: Optional[UserPendingChanges]
    Username: _string


class DescribeUserRequest(ServiceRequest):
    BrokerId: _string
    Username: _string


class DescribeUserResponse(TypedDict, total=False):
    BrokerId: Optional[_string]
    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Pending: Optional[UserPendingChanges]
    Username: Optional[_string]


class Error(TypedDict, total=False):
    """Returns information about an error."""

    ErrorAttribute: Optional[_string]
    Message: Optional[_string]


_listOfBrokerSummary = List[BrokerSummary]


class ListBrokersOutput(TypedDict, total=False):
    BrokerSummaries: Optional[_listOfBrokerSummary]
    NextToken: Optional[_string]


class ListBrokersRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]


class ListBrokersResponse(TypedDict, total=False):
    BrokerSummaries: Optional[_listOfBrokerSummary]
    NextToken: Optional[_string]


_listOfConfigurationRevision = List[ConfigurationRevision]


class ListConfigurationRevisionsOutput(TypedDict, total=False):
    """Returns a list of all revisions for the specified configuration."""

    ConfigurationId: Optional[_string]
    MaxResults: Optional[_integer]
    NextToken: Optional[_string]
    Revisions: Optional[_listOfConfigurationRevision]


class ListConfigurationRevisionsRequest(ServiceRequest):
    ConfigurationId: _string
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]


class ListConfigurationRevisionsResponse(TypedDict, total=False):
    ConfigurationId: Optional[_string]
    MaxResults: Optional[_integer]
    NextToken: Optional[_string]
    Revisions: Optional[_listOfConfigurationRevision]


_listOfConfiguration = List[Configuration]


class ListConfigurationsOutput(TypedDict, total=False):
    """Returns a list of all configurations."""

    Configurations: Optional[_listOfConfiguration]
    MaxResults: Optional[_integer]
    NextToken: Optional[_string]


class ListConfigurationsRequest(ServiceRequest):
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]


class ListConfigurationsResponse(TypedDict, total=False):
    Configurations: Optional[_listOfConfiguration]
    MaxResults: Optional[_integer]
    NextToken: Optional[_string]


class ListTagsRequest(ServiceRequest):
    ResourceArn: _string


class ListTagsResponse(TypedDict, total=False):
    Tags: Optional[_mapOf__string]


class ListUsersOutput(TypedDict, total=False):
    """Returns a list of all ActiveMQ users."""

    BrokerId: _string
    MaxResults: _integerMin5Max100
    NextToken: Optional[_string]
    Users: _listOfUserSummary


class ListUsersRequest(ServiceRequest):
    BrokerId: _string
    MaxResults: Optional[MaxResults]
    NextToken: Optional[_string]


class ListUsersResponse(TypedDict, total=False):
    BrokerId: Optional[_string]
    MaxResults: Optional[_integerMin5Max100]
    NextToken: Optional[_string]
    Users: Optional[_listOfUserSummary]


class RebootBrokerRequest(ServiceRequest):
    BrokerId: _string


class RebootBrokerResponse(TypedDict, total=False):
    pass


class SanitizationWarning(TypedDict, total=False):
    """Returns information about the XML element or attribute that was
    sanitized in the configuration.
    """

    AttributeName: Optional[_string]
    ElementName: Optional[_string]
    Reason: SanitizationWarningReason


class Tags(TypedDict, total=False):
    """A map of the key-value pairs for the resource tag."""

    Tags: Optional[_mapOf__string]


class UpdateBrokerInput(TypedDict, total=False):
    """Updates the broker using the specified properties."""

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: Optional[_boolean]
    Configuration: Optional[ConfigurationId]
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataInput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    SecurityGroups: Optional[_listOf__string]


class UpdateBrokerOutput(TypedDict, total=False):
    """Returns information about the updated broker."""

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: Optional[_boolean]
    BrokerId: _string
    Configuration: Optional[ConfigurationId]
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataOutput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    SecurityGroups: Optional[_listOf__string]


class UpdateBrokerRequest(ServiceRequest):
    """Updates the broker using the specified properties."""

    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: Optional[_boolean]
    BrokerId: _string
    Configuration: Optional[ConfigurationId]
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataInput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    SecurityGroups: Optional[_listOf__string]


class UpdateBrokerResponse(TypedDict, total=False):
    AuthenticationStrategy: Optional[AuthenticationStrategy]
    AutoMinorVersionUpgrade: Optional[_boolean]
    BrokerId: Optional[_string]
    Configuration: Optional[ConfigurationId]
    EngineVersion: Optional[_string]
    HostInstanceType: Optional[_string]
    LdapServerMetadata: Optional[LdapServerMetadataOutput]
    Logs: Optional[Logs]
    MaintenanceWindowStartTime: Optional[WeeklyStartTime]
    SecurityGroups: Optional[_listOf__string]


class UpdateConfigurationInput(TypedDict, total=False):
    """Updates the specified configuration."""

    Data: _string
    Description: Optional[_string]


_listOfSanitizationWarning = List[SanitizationWarning]


class UpdateConfigurationOutput(TypedDict, total=False):
    """Returns information about the updated configuration."""

    Arn: _string
    Created: _timestampIso8601
    Id: _string
    LatestRevision: Optional[ConfigurationRevision]
    Name: _string
    Warnings: Optional[_listOfSanitizationWarning]


class UpdateConfigurationRequest(ServiceRequest):
    """Updates the specified configuration."""

    ConfigurationId: _string
    Data: _string
    Description: Optional[_string]


class UpdateConfigurationResponse(TypedDict, total=False):
    Arn: Optional[_string]
    Created: Optional[_timestampIso8601]
    Id: Optional[_string]
    LatestRevision: Optional[ConfigurationRevision]
    Name: Optional[_string]
    Warnings: Optional[_listOfSanitizationWarning]


class UpdateUserInput(TypedDict, total=False):
    """Updates the information for an ActiveMQ user."""

    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Password: Optional[_string]


class UpdateUserRequest(ServiceRequest):
    """Updates the information for an ActiveMQ user."""

    BrokerId: _string
    ConsoleAccess: Optional[_boolean]
    Groups: Optional[_listOf__string]
    Password: Optional[_string]
    Username: _string


class UpdateUserResponse(TypedDict, total=False):
    pass


_long = int
_timestampUnix = datetime


class MqApi:

    service = "mq"
    version = "2017-11-27"

    @handler("CreateBroker")
    def create_broker(
        self,
        context: RequestContext,
        engine_version: _string,
        host_instance_type: _string,
        auto_minor_version_upgrade: _boolean,
        users: _listOfUser,
        broker_name: _string,
        deployment_mode: DeploymentMode,
        engine_type: EngineType,
        publicly_accessible: _boolean,
        authentication_strategy: AuthenticationStrategy = None,
        configuration: ConfigurationId = None,
        creator_request_id: _string = None,
        encryption_options: EncryptionOptions = None,
        ldap_server_metadata: LdapServerMetadataInput = None,
        logs: Logs = None,
        maintenance_window_start_time: WeeklyStartTime = None,
        security_groups: _listOf__string = None,
        storage_type: BrokerStorageType = None,
        subnet_ids: _listOf__string = None,
        tags: _mapOf__string = None,
    ) -> CreateBrokerResponse:
        """Creates a broker. Note: This API is asynchronous.

        To create a broker, you must either use the AmazonMQFullAccess IAM
        policy or include the following EC2 permissions in your IAM policy.

        -  ec2:CreateNetworkInterface

           This permission is required to allow Amazon MQ to create an elastic
           network interface (ENI) on behalf of your account.

        -  ec2:CreateNetworkInterfacePermission

           This permission is required to attach the ENI to the broker instance.

        -  ec2:DeleteNetworkInterface

        -  ec2:DeleteNetworkInterfacePermission

        -  ec2:DetachNetworkInterface

        -  ec2:DescribeInternetGateways

        -  ec2:DescribeNetworkInterfaces

        -  ec2:DescribeNetworkInterfacePermissions

        -  ec2:DescribeRouteTables

        -  ec2:DescribeSecurityGroups

        -  ec2:DescribeSubnets

        -  ec2:DescribeVpcs

        For more information, see `Create an IAM User and Get Your AWS
        Credentials <https://docs.aws.amazon.com//amazon-mq/latest/developer-guide/amazon-mq-setting-up.html#create-iam-user>`__
        and `Never Modify or Delete the Amazon MQ Elastic Network
        Interface <https://docs.aws.amazon.com/amazon-mq/latest/developer-guide/connecting-to-amazon-mq.html#never-modify-delete-elastic-network-interface>`__
        in the *Amazon MQ Developer Guide*.

        :param engine_version: Required.
        :param host_instance_type: Required.
        :param auto_minor_version_upgrade: Enables automatic upgrades to new minor versions for brokers, as new
        versions are released and supported by Amazon MQ.
        :param users: Required.
        :param broker_name: Required.
        :param deployment_mode: Required.
        :param engine_type: Required.
        :param publicly_accessible: Enables connections from applications outside of the VPC that hosts the
        broker's subnets.
        :param authentication_strategy: Optional.
        :param configuration: A list of information about the configuration.
        :param creator_request_id: The unique ID that the requester receives for the created broker.
        :param encryption_options: Encryption options for the broker.
        :param ldap_server_metadata: Optional.
        :param logs: Enables Amazon CloudWatch logging for brokers.
        :param maintenance_window_start_time: The parameters that determine the WeeklyStartTime.
        :param security_groups: The list of rules (1 minimum, 125 maximum) that authorize connections to
        brokers.
        :param storage_type: The broker's storage type.
        :param subnet_ids: The list of groups that define which subnets and IP ranges the broker
        can use from different Availability Zones.
        :param tags: Create tags when creating the broker.
        :returns: CreateBrokerResponse
        :raises BadRequestException:
        :raises UnauthorizedException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("CreateConfiguration")
    def create_configuration(
        self,
        context: RequestContext,
        engine_version: _string,
        engine_type: EngineType,
        name: _string,
        authentication_strategy: AuthenticationStrategy = None,
        tags: _mapOf__string = None,
    ) -> CreateConfigurationResponse:
        """Creates a new configuration for the specified configuration name. Amazon
        MQ uses the default configuration (the engine type and version).

        :param engine_version: Required.
        :param engine_type: Required.
        :param name: Required.
        :param authentication_strategy: Optional.
        :param tags: Create tags when creating the configuration.
        :returns: CreateConfigurationResponse
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("CreateTags")
    def create_tags(
        self, context: RequestContext, resource_arn: _string, tags: _mapOf__string = None
    ) -> None:
        """Add a tag to a resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource tag.
        :param tags: The key-value pair for the resource tag.
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("CreateUser")
    def create_user(
        self,
        context: RequestContext,
        username: _string,
        broker_id: _string,
        password: _string,
        console_access: _boolean = None,
        groups: _listOf__string = None,
    ) -> CreateUserResponse:
        """Creates an ActiveMQ user.

        :param username: The username of the ActiveMQ user.
        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :param password: Required.
        :param console_access: Enables access to the ActiveMQ Web Console for the ActiveMQ user.
        :param groups: The list of groups (20 maximum) to which the ActiveMQ user belongs.
        :returns: CreateUserResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DeleteBroker")
    def delete_broker(self, context: RequestContext, broker_id: _string) -> DeleteBrokerResponse:
        """Deletes a broker. Note: This API is asynchronous.

        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :returns: DeleteBrokerResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DeleteTags")
    def delete_tags(
        self, context: RequestContext, tag_keys: _listOf__string, resource_arn: _string
    ) -> None:
        """Removes a tag from a resource.

        :param tag_keys: An array of tag keys to delete.
        :param resource_arn: The Amazon Resource Name (ARN) of the resource tag.
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DeleteUser")
    def delete_user(
        self, context: RequestContext, username: _string, broker_id: _string
    ) -> DeleteUserResponse:
        """Deletes an ActiveMQ user.

        :param username: The username of the ActiveMQ user.
        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :returns: DeleteUserResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeBroker")
    def describe_broker(
        self, context: RequestContext, broker_id: _string
    ) -> DescribeBrokerResponse:
        """Returns information about the specified broker.

        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :returns: DescribeBrokerResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeBrokerEngineTypes")
    def describe_broker_engine_types(
        self,
        context: RequestContext,
        engine_type: _string = None,
        max_results: MaxResults = None,
        next_token: _string = None,
    ) -> DescribeBrokerEngineTypesResponse:
        """Describe available engine types and versions.

        :param engine_type: Filter response by engine type.
        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :returns: DescribeBrokerEngineTypesResponse
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeBrokerInstanceOptions")
    def describe_broker_instance_options(
        self,
        context: RequestContext,
        engine_type: _string = None,
        host_instance_type: _string = None,
        max_results: MaxResults = None,
        next_token: _string = None,
        storage_type: _string = None,
    ) -> DescribeBrokerInstanceOptionsResponse:
        """Describe available broker instance options.

        :param engine_type: Filter response by engine type.
        :param host_instance_type: Filter response by host instance type.
        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :param storage_type: Filter response by storage type.
        :returns: DescribeBrokerInstanceOptionsResponse
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeConfiguration")
    def describe_configuration(
        self, context: RequestContext, configuration_id: _string
    ) -> DescribeConfigurationResponse:
        """Returns information about the specified configuration.

        :param configuration_id: The unique ID that Amazon MQ generates for the configuration.
        :returns: DescribeConfigurationResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeConfigurationRevision")
    def describe_configuration_revision(
        self, context: RequestContext, configuration_revision: _string, configuration_id: _string
    ) -> DescribeConfigurationRevisionResponse:
        """Returns the specified configuration revision for the specified
        configuration.

        :param configuration_revision: The revision of the configuration.
        :param configuration_id: The unique ID that Amazon MQ generates for the configuration.
        :returns: DescribeConfigurationRevisionResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("DescribeUser")
    def describe_user(
        self, context: RequestContext, username: _string, broker_id: _string
    ) -> DescribeUserResponse:
        """Returns information about an ActiveMQ user.

        :param username: The username of the ActiveMQ user.
        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :returns: DescribeUserResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("ListBrokers")
    def list_brokers(
        self, context: RequestContext, max_results: MaxResults = None, next_token: _string = None
    ) -> ListBrokersResponse:
        """Returns a list of all brokers.

        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :returns: ListBrokersResponse
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("ListConfigurationRevisions")
    def list_configuration_revisions(
        self,
        context: RequestContext,
        configuration_id: _string,
        max_results: MaxResults = None,
        next_token: _string = None,
    ) -> ListConfigurationRevisionsResponse:
        """Returns a list of all revisions for the specified configuration.

        :param configuration_id: The unique ID that Amazon MQ generates for the configuration.
        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :returns: ListConfigurationRevisionsResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("ListConfigurations")
    def list_configurations(
        self, context: RequestContext, max_results: MaxResults = None, next_token: _string = None
    ) -> ListConfigurationsResponse:
        """Returns a list of all configurations.

        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :returns: ListConfigurationsResponse
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("ListTags")
    def list_tags(self, context: RequestContext, resource_arn: _string) -> ListTagsResponse:
        """Lists tags for a resource.

        :param resource_arn: The Amazon Resource Name (ARN) of the resource tag.
        :returns: ListTagsResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("ListUsers")
    def list_users(
        self,
        context: RequestContext,
        broker_id: _string,
        max_results: MaxResults = None,
        next_token: _string = None,
    ) -> ListUsersResponse:
        """Returns a list of all ActiveMQ users.

        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :param max_results: The maximum number of brokers that Amazon MQ can return per page (20 by
        default).
        :param next_token: The token that specifies the next page of results Amazon MQ should
        return.
        :returns: ListUsersResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("RebootBroker")
    def reboot_broker(self, context: RequestContext, broker_id: _string) -> RebootBrokerResponse:
        """Reboots a broker. Note: This API is asynchronous.

        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :returns: RebootBrokerResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("UpdateBroker")
    def update_broker(
        self,
        context: RequestContext,
        broker_id: _string,
        authentication_strategy: AuthenticationStrategy = None,
        auto_minor_version_upgrade: _boolean = None,
        configuration: ConfigurationId = None,
        engine_version: _string = None,
        host_instance_type: _string = None,
        ldap_server_metadata: LdapServerMetadataInput = None,
        logs: Logs = None,
        maintenance_window_start_time: WeeklyStartTime = None,
        security_groups: _listOf__string = None,
    ) -> UpdateBrokerResponse:
        """Adds a pending configuration change to a broker.

        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :param authentication_strategy: Optional.
        :param auto_minor_version_upgrade: Enables automatic upgrades to new minor versions for brokers, as new
        versions are released and supported by Amazon MQ.
        :param configuration: A list of information about the configuration.
        :param engine_version: The broker engine version.
        :param host_instance_type: The broker's host instance type to upgrade to.
        :param ldap_server_metadata: Optional.
        :param logs: Enables Amazon CloudWatch logging for brokers.
        :param maintenance_window_start_time: The parameters that determine the WeeklyStartTime.
        :param security_groups: The list of security groups (1 minimum, 5 maximum) that authorizes
        connections to brokers.
        :returns: UpdateBrokerResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("UpdateConfiguration")
    def update_configuration(
        self,
        context: RequestContext,
        configuration_id: _string,
        data: _string,
        description: _string = None,
    ) -> UpdateConfigurationResponse:
        """Updates the specified configuration.

        :param configuration_id: The unique ID that Amazon MQ generates for the configuration.
        :param data: Required.
        :param description: The description of the configuration.
        :returns: UpdateConfigurationResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError

    @handler("UpdateUser")
    def update_user(
        self,
        context: RequestContext,
        username: _string,
        broker_id: _string,
        console_access: _boolean = None,
        groups: _listOf__string = None,
        password: _string = None,
    ) -> UpdateUserResponse:
        """Updates the information for an ActiveMQ user.

        :param username: The username of the ActiveMQ user.
        :param broker_id: The unique ID that Amazon MQ generates for the broker.
        :param console_access: Enables access to the the ActiveMQ Web Console for the ActiveMQ user.
        :param groups: The list of groups (20 maximum) to which the ActiveMQ user belongs.
        :param password: The password of the user.
        :returns: UpdateUserResponse
        :raises NotFoundException:
        :raises BadRequestException:
        :raises InternalServerErrorException:
        :raises ConflictException:
        :raises ForbiddenException:
        """
        raise NotImplementedError



import os
from localstack import config
from localstack_ext.bootstrap.pods.server.plugins import StateLifecyclePlugin
RDS_DATA_ROOT='rds-data'
class RDSPersistencePlugin(StateLifecyclePlugin):
	name='rds';service='rds'
	def get_assets_location(A):return os.path.join(config.dirs.data,RDS_DATA_ROOT)
	def inject_assets(B,pod_asset_directory):from localstack_ext.services.rds.provider import restore_backend_state as A;super().inject_assets(pod_asset_directory);A()


_O='lambda'
_N='cloudwatch'
_M='timestream-query'
_L='timestream-write'
_K='sagemaker-runtime'
_J='redshift-data'
_I='rds-data'
_H='qldb-session'
_G='mediastore-data'
_F='iot-data'
_E='cognito-idp'
_D='cognito-identity'
_C='application-autoscaling'
_B='s3'
_A='eks'
import functools
from localstack.aws.forwarder import HttpFallbackDispatcher
from localstack.aws.proxy import AwsApiListener
from localstack.config import is_env_true
from localstack.constants import ENV_PRO_ACTIVATED
from localstack.services.moto import MotoFallbackDispatcher
from localstack.services.plugins import Service,aws_provider
pro_aws_provider=functools.partial(aws_provider,name='pro',should_load=lambda:is_env_true(ENV_PRO_ACTIVATED))
@pro_aws_provider()
def acm():A='acm';from localstack.services.acm.provider import AcmProvider as B;from localstack.services.moto import MotoFallbackDispatcher as C;D=B();return Service(A,listener=AwsApiListener(A,C(D)))
@pro_aws_provider()
def amplify():B='amplify';from localstack_ext.services.amplify.provider import AmplifyProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def apigatewaymanagementapi():A='apigatewaymanagementapi';from localstack_ext.services.apigateway.provider_mgmtapi import ApigatewaymanagementapiProvider as B;C=B();D=AwsApiListener(A,C);return Service(A,listener=D)
@pro_aws_provider()
def apigatewayv2():B='apigatewayv2';from localstack_ext.services.apigateway.provider_v2 import ApiGatewayV2Provider as C;A=C();D=AwsApiListener(B,A);return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def appconfig():B='appconfig';from localstack_ext.services.appconfig.provider import AppconfigProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider(api=_C)
def application_autoscaling():from localstack_ext.services.applicationautoscaling.provider import ApplicationAutoscalingProvider as B;A=B();return Service(_C,listener=AwsApiListener(_C,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def appsync():B='appsync';from localstack_ext.services.appsync.provider import AppSyncProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def athena():B='athena';from localstack_ext.services.athena.provider import AthenaProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def autoscaling():B='autoscaling';from localstack_ext.services.autoscaling.provider import AutoscalingProvider as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider(should_load=lambda:is_env_true(ENV_PRO_ACTIVATED)and is_env_true('AZURE'))
def azure():from localstack_ext.services.azure import azure_starter as A;return Service('azure',start=A.start_azure)
@pro_aws_provider()
def backup():B='backup';from localstack_ext.services.backup.provider import BackupProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def batch():A='batch';from localstack_ext.services.batch.provider import BatchProvider as B;C=B();D=AwsApiListener(A,MotoFallbackDispatcher(C));return Service(A,listener=D)
@pro_aws_provider()
def ce():from localstack_ext.services.costexplorer.provider import CeProvider as B;A=B();return Service('ce',listener=AwsApiListener('ce',A),lifecycle_hook=A)
@pro_aws_provider()
def cloudfront():B='cloudfront';from localstack_ext.services.cloudfront.provider import CloudFrontProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def cloudtrail():B='cloudtrail';from localstack_ext.services.cloudtrail.provider import CloudtrailProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def codecommit():B='codecommit';from localstack_ext.services.codecommit.provider import CodecommitProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_D)
def cognito_identity():from localstack_ext.services.cognito_identity.provider import CognitoIdentityProvider as B;A=B();C=AwsApiListener(_D,MotoFallbackDispatcher(A));return Service(_D,listener=C,lifecycle_hook=A)
@pro_aws_provider(api=_E)
def cognito_idp():from localstack_ext.services.cognito_idp.provider import CognitoIdpProvider as B;A=B();return Service(_E,listener=AwsApiListener(_E,A),lifecycle_hook=A)
@pro_aws_provider()
def docdb():from localstack_ext.services.docdb import docdb_api as A;return Service('docdb',start=A.start_docdb)
@pro_aws_provider()
def ec2():B='ec2';from localstack_ext.services.ec2.provider import Ec2Provider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def ecr():B='ecr';from localstack_ext.services.ecr.provider import EcrProvider as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def ecs():B='ecs';from localstack_ext.services.ecs.provider import ECSProvider as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def efs():B='efs';from localstack_ext.services.efs.provider import EfsProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def elasticache():B='elasticache';from localstack_ext.services.elasticache.provider import ElasticacheProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def elasticbeanstalk():B='elasticbeanstalk';from localstack_ext.services.elasticbeanstalk.provider import ElasticBeanstalkProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def elb():B='elb';from localstack_ext.services.elb.provider import ElbProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def elbv2():B='elbv2';from localstack_ext.services.elbv2.provider import Elbv2Provider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def eks():from localstack_ext.services.eks.provider import EksProvider as B;A=B();return Service(_A,listener=AwsApiListener(_A,A),lifecycle_hook=A)
@pro_aws_provider()
def emr():B='emr';from localstack_ext.services.emr.provider import EmrProvider as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def fis():B='fis';from localstack_ext.services.fis.provider import FisProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def glacier():B='glacier';from localstack_ext.services.glacier.provider import GlacierProvider as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def glue():B='glue';from localstack_ext.services.glue.provider import GlueProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def iot():B='iot';from localstack_ext.services.iot.provider import IotProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_F)
def iot_data():from localstack_ext.services.iot_data.provider import IotDataProvider as B;A=B();C=AwsApiListener(_F,MotoFallbackDispatcher(A));return Service(_F,listener=C,lifecycle_hook=A)
@pro_aws_provider()
def iotanalytics():B='iotanalytics';from localstack_ext.services.iotanalytics.provider import IotAnalyticsProvider as C;A=C();D=AwsApiListener(B,A);return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def iotwireless():B='iotwireless';from localstack_ext.services.iotwireless.provider import IotWirelessProvider as C;A=C();D=AwsApiListener(B,A);return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def kafka():A='kafka';from localstack_ext.services.kafka.provider import KafkaProvider as B;C=B();return Service(A,listener=AwsApiListener(A,C))
@pro_aws_provider()
def kinesisanalytics():B='kinesisanalytics';from localstack.aws.proxy import AwsApiListener as C;from localstack_ext.services.kinesisanalytics.provider import KinesisAnalyticsProvider as D;A=D();return Service(B,listener=C(B,A),lifecycle_hook=A)
@pro_aws_provider()
def kinesisanalyticsv2():B='kinesisanalyticsv2';from localstack.aws.proxy import AwsApiListener as C;from localstack_ext.services.kinesisanalyticsv2.provider import KinesisAnalyticsV2Provider as D;A=D();return Service(B,listener=C(B,A),lifecycle_hook=A)
@pro_aws_provider()
def kinesis():B='kinesis';from localstack.services.kinesis.provider import KinesisProvider as C;A=C();D=AwsApiListener(B,HttpFallbackDispatcher(A,A.get_forward_url));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def lakeformation():B='lakeformation';from localstack_ext.services.lakeformation.provider import LakeFormationProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def logs():B='logs';from localstack_ext.services.logs.provider import LogsProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def mediastore():B='mediastore';from localstack_ext.services.mediastore.provider import MediastoreProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider(api=_G)
def mediastore_data():from localstack_ext.services.mediastore.provider import MediaStoreDataProvider as B;A=B();return Service(_G,listener=AwsApiListener(_G,A),lifecycle_hook=A)
@pro_aws_provider()
def mwaa():B='mwaa';from localstack_ext.services.mwaa.provider import MwaaProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def neptune():from localstack_ext.services.neptune import neptune_api as A;return Service('neptune',start=A.start_neptune)
@pro_aws_provider()
def organizations():B='organizations';from localstack_ext.services.organizations.provider import OrganizationsProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def qldb():B='qldb';from localstack_ext.services.qldb.provider import QldbProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider(api=_H)
def qldb_session():from localstack_ext.services.qldb.provider import QldbSessionProvider as B;A=B();return Service(_H,listener=AwsApiListener(_H,A),lifecycle_hook=A)
@pro_aws_provider()
def rds():B='rds';from localstack_ext.services.rds.provider import RdsProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_I)
def rds_data():from localstack_ext.services.rds_data.provider import RdsDataProvider as A;B=A();return Service(_I,listener=AwsApiListener(_I,B))
@pro_aws_provider()
def redshift():B='redshift';from localstack_ext.services.redshift.provider import RedshiftProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_J)
def redshift_data():from localstack_ext.services.redshift.provider import RedshiftDataProvider as A;B=A();C=AwsApiListener(_J,B);return Service(_J,listener=C)
@pro_aws_provider()
def sagemaker():A='sagemaker';from localstack_ext.services.sagemaker.provider import SagemakerProvider as B;C=B();D=AwsApiListener(A,MotoFallbackDispatcher(C));return Service(A,listener=D)
@pro_aws_provider(api=_K)
def sagemaker_runtime():from localstack_ext.services.sagemaker.provider import SageMakerRuntimeProvider as A;B=A();C=AwsApiListener(_K,B);return Service(_K,listener=C)
@pro_aws_provider()
def serverlessrepo():B='serverlessrepo';from localstack_ext.services.serverlessrepo.provider import ServerlessrepoProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def servicediscovery():B='servicediscovery';from localstack_ext.services.servicediscovery.provider import ServicediscoveryProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def ssm():B='ssm';from localstack_ext.services.ssm.provider import SsmProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_L)
def timestream_write():from localstack_ext.services.timestream.provider import TimestreamWriteProvider as A;B=A();return Service(_L,listener=AwsApiListener(_L,B))
@pro_aws_provider(api=_M)
def timestream_query():from localstack_ext.services.timestream.provider import TimestreamQueryProvider as A;B=A();return Service(_M,listener=AwsApiListener(_M,B))
@pro_aws_provider()
def transfer():B='transfer';from localstack_ext.services.transfer.provider import TransferProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def xray():A='xray';from localstack_ext.services.xray.provider import XrayProvider as B;C=B();D=AwsApiListener(A,MotoFallbackDispatcher(C));return Service(A,listener=D)
@pro_aws_provider()
def apigateway():B='apigateway';from localstack_ext.services.apigateway.apigateway_extended import ApigatewayExtProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider(api=_O)
def awslambda():from localstack.services.awslambda import lambda_starter as A;from localstack_ext.services.awslambda.lambda_extended import patch_lambda as B;B();return Service(_O,start=A.start_lambda,stop=A.stop_lambda,check=A.check_lambda)
@pro_aws_provider(api=_N)
def cloudwatch():from localstack_ext.services.cloudwatch.provider import CloudwatchProvider as B;A=B();return Service(_N,listener=AwsApiListener(_N,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def cloudformation():A='cloudformation';from localstack.services.cloudformation.provider import CloudformationProvider as B;from localstack_ext.services.cloudformation import cloudformation_extended as C;C.patch_cloudformation();D=B();return Service(A,listener=AwsApiListener(A,D))
@pro_aws_provider()
def dynamodb():B='dynamodb';from localstack_ext.services.dynamodb.provider import DynamoDBProviderExt as C;A=C();D=AwsApiListener(B,HttpFallbackDispatcher(A,A.get_forward_url));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def dynamodbstreams():B='dynamodbstreams';from localstack_ext.services.dynamodbstreams.provider import DynamoDBStreamsExtProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def events():B='events';from localstack.services.moto import MotoFallbackDispatcher as C;from localstack_ext.services.events.provider import EventsProviderPro as D;A=D();return Service(B,listener=AwsApiListener(B,C(A)),lifecycle_hook=A)
@pro_aws_provider()
def mq():from localstack_ext.services.mq.provider import MQProvider as B;A=B();return Service('mq',listener=AwsApiListener('mq',MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def iam():B='iam';from localstack_ext.services.iam.provider import IamProviderExt as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def kms():from localstack.services.providers import kms as A;from localstack_ext.services.kms import kms_extended as B;B.patch_kms();return A.factory.__wrapped__()
@pro_aws_provider()
def opensearch():B='opensearch';from localstack_ext.services.opensearch.provider import OpensearchProvider as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def route53():A='route53';from localstack_ext.services.route53.provider import Route53ProviderPro as B;C=B();return Service(A,listener=AwsApiListener(A,MotoFallbackDispatcher(C)))
@pro_aws_provider(api=_B,name='asf_pro')
def s3_asf():from localstack_ext.services.s3.provider import S3ProviderPro as B;A=B();return Service(_B,listener=AwsApiListener(_B,MotoFallbackDispatcher(A)),lifecycle_hook=A,backend_state_lifecycle=A)
@pro_aws_provider()
def s3():from localstack.services.s3 import s3_listener as B,s3_starter as A;from localstack_ext.services.s3 import s3_extended as C;C.patch_s3();return Service(_B,listener=B.UPDATE_S3,start=A.start_s3,check=A.check_s3)
@pro_aws_provider()
def secretsmanager():B='secretsmanager';from localstack_ext.services.secretsmanager.secretsmanager_extended import SecretsmanagerProviderExt as C;A=C();return Service(B,listener=AwsApiListener(B,MotoFallbackDispatcher(A)),lifecycle_hook=A)
@pro_aws_provider()
def ses():B='ses';from localstack_ext.services.ses.provider import SesProvider as C;A=C();D=AwsApiListener(B,MotoFallbackDispatcher(A));return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def sesv2():B='sesv2';from localstack_ext.services.sesv2.provider import Sesv2Provider as C;A=C();D=AwsApiListener(B,A);return Service(B,listener=D,lifecycle_hook=A)
@pro_aws_provider()
def sns():B='sns';from localstack_ext.services.sns.provider import SnsProviderExt as C;A=C();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def sqs():B='sqs';from localstack.services import edge;from localstack.services.sqs import query_api as C;from localstack_ext.services.sqs.provider import SqsProvider as D;C.register(edge.ROUTER);A=D();return Service(B,listener=AwsApiListener(B,A),lifecycle_hook=A)
@pro_aws_provider()
def stepfunctions():B='stepfunctions';from localstack.services.stepfunctions.provider import StepFunctionsProvider as C;from localstack_ext.services.stepfunctions.provider import patch_stepfunctions as D;D();A=C();E=AwsApiListener(B,HttpFallbackDispatcher(A,A.get_forward_url));return Service(B,listener=E,lifecycle_hook=A)
@pro_aws_provider()
def sts():from localstack.services.providers import sts as A;from localstack_ext.services.sts import sts_extended as B;B.patch_sts();return A.factory.__wrapped__()
@pro_aws_provider(name='mock',api=_A)
def eks_mock():from localstack_ext.services.eks.provider import EksMockProvider as B;A=B();return Service(_A,listener=AwsApiListener(_A,A),lifecycle_hook=A)